#include <qgroupbox.h>
#include <kkeyconf.h>
#include <qlayout.h>
#include <kapp.h>
#include <kconfig.h>
#include <kmsgbox.h>
#include <qvalidator.h>
#include <klocale.h>

#include "dialogs.h"

KeySetup::KeySetup(SOptions *opt,QWidget *parent,const char *name)
      :QDialog(parent,name,true)
{
   int i,p;

   waitForKey=-1;
   options=opt;

   setCaption(klocale->translate("Key Setup"));

   for(p=0;p<2;p++)
      for(i=0;i<PlayerKeyNum;i++)
         key[p][i]=opt->playerKey[p][i];
   for(i=0;i<FunctionKeyNum;i++)
      fkey[i]=opt->functionKey[i];

   QVBoxLayout *topLayout=new QVBoxLayout(this,10);

   QGroupBox *bplayer[2];
   bplayer[0]=new QGroupBox(klocale->translate("Red Player"),this);
   bplayer[1]=new QGroupBox(klocale->translate("Blue Player"),this);
 
   for(p=0;p<2;p++)
   {
      button[p][PlayerKeyLeft]=new QPushButton(klocale->translate("Rotate Left"),bplayer[p]);
      button[p][PlayerKeyRight]=new QPushButton(klocale->translate("Rotate Right"),bplayer[p]);
      button[p][PlayerKeyAcc]=new QPushButton(klocale->translate("Accelerate"),bplayer[p]);
      button[p][PlayerKeyShot]=new QPushButton(klocale->translate("Shot"),bplayer[p]);
      button[p][PlayerKeyMine]=new QPushButton(klocale->translate("Mine"),bplayer[p]);
      for(i=0;i<PlayerKeyNum;i++)
      {
         keyName[p][i]=new QLabel(keyToString((unsigned)key[p][i]),bplayer[p]);
         keyName[p][i]->setLineWidth(1);
         keyName[p][i]->setFrameStyle(QFrame::Sunken|QFrame::Panel);
      }
   }

   QGridLayout *grid[2];
   for(p=0;p<2;p++)
   {
      grid[p]=new QGridLayout(bplayer[p],PlayerKeyNum+1,2,10);
      grid[p]->addRowSpacing(0,5);
      grid[p]->setRowStretch(0,0);
      for(i=0;i<PlayerKeyNum;i++)
      {
         button[p][i]->setToggleButton(true);
         grid[p]->addWidget(button[p][i],i+1,0);
         grid[p]->addWidget(keyName[p][i],i+1,1);
         grid[p]->setRowStretch(i+1,1);
      }
      grid[p]->activate();
   }

   connect(button[0][PlayerKeyLeft],SIGNAL(clicked()),SLOT(keyLeft1()));
   connect(button[0][PlayerKeyRight],SIGNAL(clicked()),SLOT(keyRight1()));
   connect(button[0][PlayerKeyAcc],SIGNAL(clicked()),SLOT(keyAcc1()));
   connect(button[0][PlayerKeyShot],SIGNAL(clicked()),SLOT(keyShot1()));
   connect(button[0][PlayerKeyMine],SIGNAL(clicked()),SLOT(keyMine1()));
   connect(button[1][PlayerKeyLeft],SIGNAL(clicked()),SLOT(keyLeft2()));
   connect(button[1][PlayerKeyRight],SIGNAL(clicked()),SLOT(keyRight2()));
   connect(button[1][PlayerKeyAcc],SIGNAL(clicked()),SLOT(keyAcc2()));
   connect(button[1][PlayerKeyShot],SIGNAL(clicked()),SLOT(keyShot2()));
   connect(button[1][PlayerKeyMine],SIGNAL(clicked()),SLOT(keyMine2()));

   QHBoxLayout *players=new QHBoxLayout;
   topLayout->addLayout(players,PlayerKeyNum);
   players->addWidget(bplayer[0]);
   players->addWidget(bplayer[1]);

   QGroupBox *bgeneral=new QGroupBox(klocale->translate("General"),this);
   QGridLayout *generalgrid=new QGridLayout(bgeneral,FunctionKeyNum+1,2,10);
   generalgrid->addRowSpacing(0,5);
   generalgrid->setRowStretch(0,0);

   fbutton[FunctionKeyStart]=new QPushButton(klocale->translate("Start/Pause"),bgeneral);
   for(i=0;i<FunctionKeyNum;i++)
   {
      fKeyName[i]=new QLabel(keyToString(fkey[i]),bgeneral);
      fKeyName[i]->setLineWidth(1);
      fKeyName[i]->setFrameStyle(QFrame::Sunken|QFrame::Panel);
      fbutton[i]->setToggleButton(true);
      generalgrid->addWidget(fbutton[i],i+1,0);
      generalgrid->addWidget(fKeyName[i],i+1,1);
      generalgrid->setRowStretch(i+1,1);
   }
   generalgrid->activate();
   topLayout->addWidget(bgeneral,FunctionKeyNum);
   connect(fbutton[FunctionKeyStart],SIGNAL(clicked()),SLOT(keyStart()));
   
   QHBoxLayout *buttons=new QHBoxLayout;
   topLayout->addLayout(buttons,1);
   ok=new QPushButton(klocale->translate("Ok"),this);
   ok->setDefault(true);
   connect(ok,SIGNAL(clicked()),SLOT(okPressed()));
   cancel=new QPushButton(klocale->translate("Cancel"),this);
   connect(cancel,SIGNAL(clicked()),SLOT(reject()));
   defaults=new QPushButton(klocale->translate("Defaults"),this);
   connect(defaults,SIGNAL(clicked()),SLOT(defaultsPressed()));
   buttons->addWidget(defaults);
   buttons->addWidget(cancel);
   buttons->addWidget(ok);
   resize(440,350);
   topLayout->activate();
}

void KeySetup::setButtons(int pl,int b)
{
   int i,p;
   for(p=0;p<2;p++)
      for(i=0;i<PlayerKeyNum;i++)
         button[p][i]->setOn((pl==p)&&(b==i));
   for(i=0;i<FunctionKeyNum;i++)
      fbutton[i]->setOn(pl==2);
   waitForKey=b;
   player=pl;
}

void KeySetup::defaultsPressed()
{
   int p,i;
   key[0][PlayerKeyLeft]=Key_S;
   key[0][PlayerKeyRight]=Key_F;
   key[0][PlayerKeyAcc]=Key_E;
   key[0][PlayerKeyShot]=Key_D;
   key[0][PlayerKeyMine]=Key_A;
   key[1][PlayerKeyLeft]=Key_Left;
   key[1][PlayerKeyRight]=Key_Right;
   key[1][PlayerKeyAcc]=Key_Up;
   key[1][PlayerKeyShot]=Key_Down;
   key[1][PlayerKeyMine]=Key_Control;
   fkey[FunctionKeyStart]=Key_Space;

   for(p=0;p<2;p++)
      for(i=0;i<PlayerKeyNum;i++)
         keyName[p][i]->setText(keyToString(key[p][i]));
   for(i=0;i<FunctionKeyNum;i++)
      fKeyName[i]->setText(keyToString(fkey[i]));
}

void KeySetup::okPressed()
{
   int p,i,j;
   KConfig *cfg;
   bool sameKey=false;

   for(i=0;i<PlayerKeyNum;i++)
   {
      for(j=i+1;j<PlayerKeyNum;j++)
         sameKey=sameKey||(key[0][i]==key[0][j]);
      for(j=0;j<PlayerKeyNum;j++)
         sameKey=sameKey||(key[0][i]==key[1][j]);
      for(j=0;j<FunctionKeyNum;j++)
         sameKey=sameKey||(key[0][i]==fkey[j]);
   }
   for(i=0;i<PlayerKeyNum;i++)
   {
      for(j=i+1;j<PlayerKeyNum;j++)
         sameKey=sameKey||(key[1][i]==key[1][j]);
      for(j=0;j<FunctionKeyNum;j++)
         sameKey=sameKey||(key[1][i]==fkey[j]);
   }
   if(sameKey)
      sameKey=KMsgBox::yesNo(this,klocale->translate("Key Setup"),
                             klocale->translate("There are multiple functions for a key\nContinue?"),
                             KMsgBox::EXCLAMATION)!=0;
   if(!sameKey)
   {
      for(p=0;p<2;p++)
         for(i=0;i<PlayerKeyNum;i++)
            options->playerKey[p][i]=key[p][i];
      for(i=0;i<FunctionKeyNum;i++)
         options->functionKey[i]=fkey[i];
      
      cfg=KApplication::getKApplication()->getConfig();
      cfg->setGroup("Player2");
      cfg->writeEntry("KeyLeft",keyToString(options->playerKey[1][PlayerKeyLeft]));
      cfg->writeEntry("KeyRight",keyToString(options->playerKey[1][PlayerKeyRight]));
      cfg->writeEntry("KeyAcc",keyToString(options->playerKey[1][PlayerKeyAcc]));
      cfg->writeEntry("KeyShot",keyToString(options->playerKey[1][PlayerKeyShot]));
      cfg->writeEntry("KeyMine",keyToString(options->playerKey[1][PlayerKeyMine]));
      cfg->setGroup("Player1");
      cfg->writeEntry("KeyLeft",keyToString(options->playerKey[0][PlayerKeyLeft]));
      cfg->writeEntry("KeyRight",keyToString(options->playerKey[0][PlayerKeyRight]));
      cfg->writeEntry("KeyAcc",keyToString(options->playerKey[0][PlayerKeyAcc]));
      cfg->writeEntry("KeyShot",keyToString(options->playerKey[0][PlayerKeyShot]));
      cfg->writeEntry("KeyMine",keyToString(options->playerKey[0][PlayerKeyMine]));
      cfg->setGroup("Game");
      cfg->writeEntry("KeyStart",keyToString(options->functionKey[FunctionKeyStart]));
      
      accept();
   }
}

void KeySetup::keyLeft1()
{
   setButtons(0,PlayerKeyLeft);
}

void KeySetup::keyRight1()
{
   setButtons(0,PlayerKeyRight);
}

void KeySetup::keyAcc1()
{
   setButtons(0,PlayerKeyAcc);
}

void KeySetup::keyShot1()
{
   setButtons(0,PlayerKeyShot);
}

void KeySetup::keyMine1()
{
   setButtons(0,PlayerKeyMine);
}

void KeySetup::keyLeft2()
{
   setButtons(1,PlayerKeyLeft);
}

void KeySetup::keyRight2()
{
   setButtons(1,PlayerKeyRight);
}

void KeySetup::keyAcc2()
{
   setButtons(1,PlayerKeyAcc);
}

void KeySetup::keyShot2()
{
   setButtons(1,PlayerKeyShot);
}

void KeySetup::keyMine2()
{
   setButtons(1,PlayerKeyMine);
}

void KeySetup::keyStart()
{
   setButtons(2,FunctionKeyStart);
}

void KeySetup::keyPressEvent(QKeyEvent *ev)
{
   if(waitForKey>=0)
   {
      if(player<2)
      {
         key[player][waitForKey]=ev->key();
         keyName[player][waitForKey]->setText(keyToString(ev->key()));
      }
      else
      {
         fkey[waitForKey]=ev->key();
         fKeyName[waitForKey]->setText(keyToString(ev->key()));
      }
      setButtons(-1,-1);
      ev->accept();
   }
   else
      QDialog::keyPressEvent(ev);
}

char ConfigSetup::LabelName[EditNum][20]=
{"Refresh time","Red Player","Blue Player",
 "Sun energy","Gravity",
 "Shot speed","Energy need","Max number","Damage",
 "Mine fuel","Energy need","Activate time","Damage","Max number",
 "Game speed",
 "Position X","Position Y","Velocity X","Velocity Y",
 "Acceleration","Energy need","Rotation speed",
 "Energy need","Crash damage"};

int ConfigSetup::Position[EditNum]=
{0,0,1,
 0,1,
 0,1,2,3,
 0,1,2,3,4,
 0,
 0,1,2,3,
 0,1,2,3,4};

enum ConfigSetup::Type ConfigSetup::VarType[EditNum]=
{VarInt,VarInt,VarInt,
 VarFloat,VarFloat,
 VarFloat,VarFloat,VarInt,VarInt,
 VarFloat,VarFloat,VarFloat,VarInt,VarInt,
 VarFloat,
 VarFloat,VarFloat,VarFloat,VarFloat,
 VarFloat,VarFloat,VarFloat,
 VarFloat,VarInt};

float ConfigSetup::EditVal[EditNum][3]=
{{10,100,33},{10,99,99},{10,99,99},
 {100,90000,9000},{0,10000,2200},
 {0.1,10,3},{0,99,10},{0,20,5},{1,100,20},
 {0,1000,65},{0,99,5},{5,100,15},{1,100,30},{0,20,3},
 {0.1,10,1},
 {-250,250,-130},{-180,180,-100},{-10,10,3},{-10,10,-1.7},
 {0,2,0.2},{0,10,1.0},{0.1,4,1},
 {0,10,0.2},{0,99,50}};

int ConfigSetup::Parent[EditNum]=
{BoxGraphic,BoxHitpoints,BoxHitpoints,
 BoxSun,BoxSun,
 BoxBullet,BoxBullet,BoxBullet,BoxBullet,
 BoxMine,BoxMine,BoxMine,BoxMine,BoxMine,
 BoxGeneral,
 BoxStart,BoxStart,BoxStart,BoxStart,
 BoxShip,BoxShip,BoxShip,BoxShip,BoxShip};

int ConfigSetup::BoxLineNum[BoxNum]={1,1,4,1,2,3,1,2,3};
char ConfigSetup::BoxName[BoxNum][20]=
{"Graphic","Hit points","Config","Sun","Bullet","Mine",
 "General","Start","Ship"};



ConfigSetup::ConfigSetup(SConfig *custom,SOptions *opt,
                         QWidget *parent,const char *name)
      :QDialog(parent,name,true)
{
   QGroupBox* group[BoxNum];
   QLabel *label[EditNum];
   QGridLayout *boxlayout[BoxNum];
   int i,j;
   
   resize(720,500);
   
   customConfig=custom;
   config=*customConfig;
   gameOptions=opt;
   options=*opt;

   for(i=0;i<=BoxConfig;i++)
      group[i]=new QGroupBox(klocale->translate(BoxName[i]),this);
   for(i=BoxSun;i<BoxNum;i++)
      group[i]=new QGroupBox(klocale->translate(BoxName[i]),group[BoxConfig]);
   for(i=0;i<BoxNum;i++)
   {
      boxlayout[i]=new QGridLayout(group[i],BoxLineNum[i]+1,i==BoxConfig?2:4,10);
      boxlayout[i]->addRowSpacing(0,5);
      for(j=0;j<BoxLineNum[i]+1;j++)
         boxlayout[i]->setRowStretch(j,j==0?0:1);
      if(i!=BoxConfig)
      {
         boxlayout[i]->setColStretch(0,7);
         boxlayout[i]->setColStretch(1,3);
         boxlayout[i]->setColStretch(2,7);
         boxlayout[i]->setColStretch(3,3);
      }
   }
   
   for(i=0;i<EditSunEnergy;i++)
   {
      label[i]=new QLabel(klocale->translate(LabelName[i]),group[Parent[i]]);
      edit[i]=new MyLineEdit(group[Parent[i]]);
      if(VarType[i]==VarInt)
         validator[i]=new QIntValidator((int)(EditVal[i][0]+0.5),
                                        (int)(EditVal[i][1]+0.5),edit[i]);
      else
         validator[i]=new QDoubleValidator(EditVal[i][0],
                                        EditVal[i][1],2,edit[i]);
      edit[i]->setValidator(validator[i]);
      edit[i]->setMaxLength(7);
      connect(edit[i],SIGNAL(textChanged(const char*)),
              SLOT(textChanged(const char*)));
      connect(edit[i],SIGNAL(gotFocus()),SLOT(gotFocus()));
      connect(edit[i],SIGNAL(lostFocus()),SLOT(lostFocus()));
   }
   configCombo=new QComboBox(false,group[BoxConfig]);
   connect(configCombo,SIGNAL(activated(int)),SLOT(configSelected(int)));
   for(i=0;i<predefinedConfigNum;i++)
      configCombo->insertItem(klocale->translate(predefinedConfigName[i]));
   configCombo->insertItem(klocale->translate("Custom"));

   for(i=EditSunEnergy;i<EditNum;i++)
   {
      label[i]=new QLabel(klocale->translate(LabelName[i]),group[Parent[i]]);
      edit[i]=new MyLineEdit(group[Parent[i]]);
      if(VarType[i]==VarInt)
         validator[i]=new QIntValidator((int)(EditVal[i][0]+0.5),
                                        (int)(EditVal[i][1]+0.5),edit[i]);
      else
         validator[i]=new QDoubleValidator(EditVal[i][0],
                                        EditVal[i][1],2,edit[i]);
      edit[i]->setValidator(validator[i]);
      edit[i]->setMaxLength(7);
      connect(edit[i],SIGNAL(textChanged(const char*)),
              SLOT(textChanged(const char*)));
      connect(edit[i],SIGNAL(gotFocus()),SLOT(gotFocus()));
      connect(edit[i],SIGNAL(lostFocus()),SLOT(lostFocus()));
   }
   
   boxlayout[BoxConfig]->setRowStretch(3,2);
   boxlayout[BoxConfig]->setRowStretch(4,3);

   QGridLayout *toplayout=new QGridLayout(this,3,2,10);
   toplayout->setRowStretch(0,1);
   toplayout->setRowStretch(1,7);
   toplayout->setRowStretch(2,1);
   toplayout->addRowSpacing(0,35);
   toplayout->addRowSpacing(1,200);
   toplayout->addWidget(group[BoxGraphic],0,0);
   toplayout->addWidget(group[BoxHitpoints],0,1);

   for(i=0;i<EditNum;i++)
   {
      boxlayout[Parent[i]]->addWidget(label[i],1+Position[i]/2,Position[i]%2*2);
      boxlayout[Parent[i]]->addWidget(edit[i],1+Position[i]/2,Position[i]%2*2+1);
   }
   boxlayout[BoxConfig]->addWidget(configCombo,1,0);
   
   boxlayout[BoxConfig]->addWidget(group[BoxSun],2,0);
   boxlayout[BoxConfig]->addWidget(group[BoxBullet],3,0);
   boxlayout[BoxConfig]->addWidget(group[BoxMine],4,0);

   boxlayout[BoxConfig]->addWidget(group[BoxGeneral],2,1);
   boxlayout[BoxConfig]->addWidget(group[BoxStart],3,1);
   boxlayout[BoxConfig]->addWidget(group[BoxShip],4,1);

   toplayout->addMultiCellWidget(group[BoxConfig],1,1,0,1);
   QHBoxLayout *blayout=new QHBoxLayout;
   toplayout->addLayout(blayout,2,1);

   help=new QPushButton(klocale->translate("Help"),this);
   connect(help,SIGNAL(clicked()),SLOT(helpPressed()));
   cancel=new QPushButton(klocale->translate("Cancel"),this);
   connect(cancel,SIGNAL(clicked()),SLOT(reject()));
   ok=new QPushButton(klocale->translate("Ok"),this);
   ok->setDefault(true);
   connect(ok,SIGNAL(clicked()),SLOT(okPressed()));

   blayout->addWidget(help);
   blayout->addWidget(cancel);
   blayout->addWidget(ok);

   message=new QLabel(this);
   message->setFrameStyle(QFrame::Panel|QFrame::Sunken);
   message->setLineWidth(1);
   toplayout->addWidget(message,2,0);

   for(i=0;i<BoxNum;i++)
      boxlayout[i]->activate();
   toplayout->activate();

   configCombo->setCurrentItem(opt->lastConfig);
   if(opt->lastConfig==predefinedConfigNum)
   {
      displayConfig(config);
      for(i=EditSunEnergy;i<EditNum;i++)
         edit[i]->setEnabled(true);
   }
   else
   {
      displayConfig(predefinedConfig[opt->lastConfig]);
      for(i=EditSunEnergy;i<EditNum;i++)
         edit[i]->setEnabled(false);
   }
   setValue(EditRefreshTime,opt->refreshTime);
   setValue(EditRedHitpoints,opt->startHitPoints[0]);
   setValue(EditBlueHitpoints,opt->startHitPoints[1]);
}

void ConfigSetup::valueChanged(int ednum,int value)
{
   switch(ednum)
   {
      case EditRefreshTime:options.refreshTime=value;break;
      case EditRedHitpoints:options.startHitPoints[0]=value;break;
      case EditBlueHitpoints:options.startHitPoints[1]=value;break;
   }
   if(options.lastConfig==predefinedConfigNum)
      switch(ednum)
      {
         case EditMaxBullets:config.maxBullets=value;break;
         case EditBulletDamage:config.bulletDamage=value;break;
         case EditMaxMines:config.maxMines=value;break;
         case EditMineDamage:config.mineDamage=value;break;
         case EditShipDamage:config.shipDamage=value;break;
      }
}

void ConfigSetup::valueChanged(int ednum,float value)
{
   if(options.lastConfig==predefinedConfigNum)
      switch(ednum)
      {
         case EditSunEnergy:config.sunEnergy=value;break;
         case EditGravity:config.gravity=value;break;
         case EditShotSpeed:config.shotSpeed=value;break;
         case EditShotEnergyNeed:config.shotEnergyNeed=value;break;
         case EditMineFuel:config.mineFuel=value;break;
         case EditMineEnergyNeed:config.mineEnergyNeed=value;break;
         case EditMineActivateTime:config.mineActivateTime=value;break;
         case EditGamespeed:config.gamespeed=value;break;
         case EditPosX:config.startPosX=value;break;
         case EditPosY:config.startPosY=value;break;
         case EditVelX:config.startVelX=value;break;
         case EditVelY:config.startVelY=value;break;
         case EditAcc:config.acc=value;break;
         case EditEnergyNeed:config.energyNeed=value;break;
         case EditRotationSpeed:config.rotationSpeed=value;break;
         case EditRotationEnergyNeed:config.rotationEnergyNeed=value;break;
      }
}

void ConfigSetup::okPressed()
{
   KConfig *cfg;
   int i;
   
   i=validateConfig(true);
   
   if(i==-1)
   {
      *customConfig=config;
      *gameOptions=options;
   
      cfg=KApplication::getKApplication()->getConfig();
      cfg->setGroup("Game");
      cfg->writeEntry("refreshTime",options.refreshTime);
      cfg->writeEntry("lastConfig",options.lastConfig);
      cfg->writeEntry("gamespeed",config.gamespeed);
      cfg->writeEntry("timeAfterKill",options.timeAfterKill);
      
      cfg->writeEntry("gravity",config.gravity);
      cfg->writeEntry("acceleration",config.acc);
      cfg->writeEntry("bulletDamage",config.bulletDamage);
      cfg->writeEntry("mineDamage",config.mineDamage);
      cfg->writeEntry("shipDamage",config.shipDamage);
      cfg->writeEntry("maxBullets",config.maxBullets);
      cfg->writeEntry("maxMines",config.maxMines);
      cfg->writeEntry("rotationSpeed",config.rotationSpeed);
      cfg->writeEntry("shotSpeed",config.shotSpeed);
      cfg->writeEntry("accEnergyNeed",config.energyNeed);
      cfg->writeEntry("rotationEnergyNeed",config.rotationEnergyNeed);
      cfg->writeEntry("sunEnergy",config.sunEnergy);
      cfg->writeEntry("mineActivateTime",config.mineActivateTime);
      cfg->writeEntry("mineFuel",config.mineFuel);
      cfg->writeEntry("shotEnergyNeed",config.shotEnergyNeed);
      cfg->writeEntry("mineEnergyNeed",config.mineEnergyNeed);
      
      cfg->writeEntry("startPosX",config.startPosX);
      cfg->writeEntry("startPosY",config.startPosY);
      cfg->writeEntry("startVelX",config.startVelX);
      cfg->writeEntry("startVelY",config.startVelY);
      
      cfg->setGroup("Player2");
      cfg->writeEntry("startHitPoints",options.startHitPoints[1]);
      cfg->setGroup("Player1");
      cfg->writeEntry("startHitPoints",options.startHitPoints[0]);
      accept();
   }
   else
   {
      edit[i]->setFocus();
      edit[i]->selectAll();
   }
}

void ConfigSetup::helpPressed()
{
   KApplication::getKApplication()->invokeHTMLHelp("kspaceduel/index-3.html","");
}

void ConfigSetup::configSelected(int num)
{
   bool update=true;
   int i;
   
   if((options.lastConfig==predefinedConfigNum)&&(num==predefinedConfigNum))
      update=false;
   if((options.lastConfig==predefinedConfigNum)&&(num!=predefinedConfigNum))
   {
      i=validateConfig(false);
      if(i!=-1)
      {
         update=false;
         edit[i]->setFocus();
         edit[i]->selectAll();
      }
   }

   if(update)
   {
      options.lastConfig=num;
      for(i=EditSunEnergy;i<EditNum;i++)
         edit[i]->setEnabled(num==predefinedConfigNum);
      if(num<predefinedConfigNum)
         displayConfig(predefinedConfig[num]);
      else
         displayConfig(config);
      message->setText("");
   }
   else
      configCombo->setCurrentItem(options.lastConfig);
}

int ConfigSetup::validateConfig(bool all)
{
   bool allvalid=true;
   bool valid[EditNum];
   int i,pos=0,firstInvalid=-1,firstEdit,lastEdit;
   QString str;

   firstEdit=all?0:EditSunEnergy;
   lastEdit=options.lastConfig==predefinedConfigNum?EditNum-1:EditSunEnergy;

   for(i=firstEdit;i<=lastEdit;i++)
   {
      str=edit[i]->text();
      valid[i]=validator[i]->validate(str,pos)==QValidator::Acceptable;
      if(!valid[i] && allvalid)
         firstInvalid=i;
      allvalid=allvalid&&valid[i];
   }

   if(!allvalid)
   {
      if(KMsgBox::yesNo(this,klocale->translate("Game Setup"),
                        klocale->translate("Not all parameters are valid\n"
                 "All invalid parameters will set to default\n"
                 "Continue?"),KMsgBox::EXCLAMATION)==1)
         return -1;
      else
         return firstInvalid;
   }
   
   return -1;
}

void ConfigSetup::displayConfig(SConfig cfg)
{
   setValue(EditSunEnergy,cfg.sunEnergy);
   setValue(EditGravity,cfg.gravity);
   
   setValue(EditShotSpeed,cfg.shotSpeed);
   setValue(EditShotEnergyNeed,cfg.shotEnergyNeed);
   setValue(EditMaxBullets,cfg.maxBullets);
   setValue(EditBulletDamage,cfg.bulletDamage);
   
   setValue(EditMineFuel,cfg.mineFuel);
   setValue(EditMineEnergyNeed,cfg.mineEnergyNeed);
   setValue(EditMineActivateTime,cfg.mineActivateTime);
   setValue(EditMineDamage,cfg.mineDamage);
   setValue(EditMaxMines,cfg.maxMines);
   
   setValue(EditGamespeed,cfg.gamespeed);
   
   setValue(EditPosX,cfg.startPosX);
   setValue(EditPosY,cfg.startPosY);
   setValue(EditVelX,cfg.startVelX);
   setValue(EditVelY,cfg.startVelY);
   
   setValue(EditAcc,cfg.acc);
   setValue(EditEnergyNeed,cfg.energyNeed);
   setValue(EditRotationSpeed,cfg.rotationSpeed);
   setValue(EditRotationEnergyNeed,cfg.rotationEnergyNeed);
   setValue(EditShipDamage,cfg.shipDamage);
}

void ConfigSetup::setValue(int ednum,int val)
{
   QString str;
   str.setNum(val);
   edit[ednum]->setText(str);
}

void ConfigSetup::setValue(int ednum,float val)
{
   QString str;
   str.setNum(val);
   edit[ednum]->setText(str);
}

void ConfigSetup::setValue(int ednum,unsigned val)
{
   QString str;
   str.setNum(val);
   edit[ednum]->setText(str);
}

void ConfigSetup::textChanged(const char *text)
{
   int num,pos=0;
   QString str(text);
   bool valid;

   if(options.lastConfig==predefinedConfigNum)
   {
      for(num=0;edit[num]!=sender();num++);
      if(validator[num]->validate(str,pos)!=QValidator::Acceptable)
      {
         if(VarType[num]==VarInt)
            valueChanged(num,(int)(EditVal[num][2]+0.5));
         else
            valueChanged(num,EditVal[num][2]);
      }
      else
      {
         if(VarType[num]==VarInt)
            valueChanged(num,str.toInt(&valid));
         else
            valueChanged(num,str.toFloat(&valid));
      }
   }
}

void ConfigSetup::gotFocus()
{
   int num;
   QString str,hu,ho;
   for(num=0;edit[num]!=sender();num++);

   if(VarType[num]==VarInt)
   {
      hu.setNum((int)(EditVal[num][0]+0.5));
      ho.setNum((int)(EditVal[num][1]+0.5));
      str.sprintf(klocale->translate("Integer between %s and %s"),
                  (const char*)hu,(const char*)ho);
   }
   else
   {
      hu.setNum(EditVal[num][0]);
      ho.setNum(EditVal[num][1]);
      str.sprintf(klocale->translate("Float between %s and %s"),
                  (const char*)hu,(const char*)ho);
   }
   message->setText(str);
}

void ConfigSetup::lostFocus()
{
   message->setText("");
}
